/******************************************************************
*  rot.c                                                          *
*                                                                 *
*  a simple implementation of the rot13 [en,de]coding scheme.     *
*                                                                 *
* copyright (C) 1997 Shawn Boyette                                *
* this software is free and is distributed under the terms of the *
* GNU General Public License.  See the file COPYING for more info *
*                                                                 *
* usage: rot13 [options] <filename>                               *
*        runs <filename> through a rot13 code routine and writes  *
*        out the crypted file                                     *
*                                                                 *
* version 0.9 - took input from prompt, coded it and printf'ed it *
* version 1.0 - runs over input file, accepts -o, -q, -h args     *
* version 1.1 - added -d arg, fixed problem with [,\,],^,_,and `  *
* version 1.2 - added -c, -u , -r args                            *
* version 1.3 - removed -c, -u, -r. Uses stdin for default input  *
*               (thanks to Kevin Turner (Acapnotic) for    *
*               this) and stdout for output.  Can be piped.       *
******************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char    *program_name;       /* name of the program */
char    delete_str[100];     /* command string for system() call */
int     user_output=0;       /* switch for -o option */
int     quiet_mode=1;        /* switch for -q option */
int     delete_flag=0;       /* switch for -d option */
int     stdio_flag=0;        /* flag for -s option */
int     checksum;            /* value of checksum */
int     char_in = 0;         /* character read from input file */
float   total_chars = 0;     /* counter for chars in file */
float   chars_conv = 0;      /* counter for converted chars */
float   percent = 0;         /* percentage converted */
FILE    *in_file;            /* input file */
FILE    *out_file;           /* output file */

main(int argc, char *argv[]) {

  /* declare functions */
  char ndcode(char translator);  /* conversion routine */
  void help(void);               /* onscreen help */

  /* tell the program it's name */
  program_name = argv[0];

  /*
   * parse command line args
   */
  while((argc > 1) && (argv[1][0] == '-')) {
    switch(argv[1][1]) {
      case 'q':
        quiet_mode = 0;
        break;
      case 'o':
        out_file = fopen(&argv[1][2], "w");
        user_output = 1;
        break;
      case 'd':
        delete_flag = 1;
        break;
      case 'h':
        help();
      default:
        (void)fprintf(stderr,"\nBad option: %s\n", argv[1]);
        help();
    }
    argv++;    /* move arg list up by 1 */
    argc--;    /* decrement arg count */
  }
  
  /* see if a file was specified. . . */
  if(argc == 1) {
    in_file = stdin;
    stdio_flag = 1;
  }

  /* If not using standard input, see if <infile> exists */
  if(!stdio_flag) {
    in_file = fopen(argv[1], "r");
    if(in_file == NULL) {
      (void)printf("Hmmm...%s does not seem to exist.\n", argv[1]);
      exit(1);
    }
  }


  /* send output to stdout by default. . . */
  if(!user_output)
    out_file = stdout;

  /* crypt file */
  while(1) {
    char_in = fgetc(in_file);
    if(char_in == EOF)
      break;
    total_chars++;
    fputc(ndcode(char_in), out_file);
  }

  /* delete input file if -d switch thrown */
  if(delete_flag) {
#ifdef unix
    (void)strcpy(delete_str, "rm ");
#endif
#ifdef __MSDOS__
    (void)strcpy(delete_str, "del ");
#endif
    (void)strcat(delete_str, argv[1]);
    system(delete_str);
  }

  /* print stats if not quiet mode */
  if(!quiet_mode) {
    if (total_chars < 1) {
      (void)printf("\nCan't do stats for < 1 chars...Bailing out!\n\n");
      exit(3);
    }
    percent = (chars_conv / total_chars) * 100.0;

    (void)printf("\nTotal characters read: %f\n", total_chars);
    (void)printf("Characters encoded   : %f\n", chars_conv);
    (void)printf("Encryption percentage: %f\n\n", percent);
  }

  (void)fclose(in_file);
  (void)fclose(out_file);
  return (0);
}



/*************************************
* ndcode - rot13 codec function      *
*                                    *
* parameters -                       *
*   translator -- char to be encoded *
*                 or decoded         *
*                                    *
* returns -                          *
*   the input character, run through *
*     rot13 coding                   *
*************************************/

char ndcode(char trans) {
  /* only encode letters... */
  if ((trans >= 65 && trans <= 90) || (trans >= 97 && trans <= 122)) {
    /*
     * rollback 26 if 'N' to 'Z' OR 'n' to 'z'
     * this will rot letters inside their own case
     * as per the example in the Hacker Jargon File 4.0
     */
    if (trans >= 78 && trans <= 90)
      trans -= 26;
    if (trans >= 110 && trans <= 122)
      trans -= 26;

    /* do rot13 encoding */
    trans += 13;
    chars_conv++;
  }
  return (trans);
}



/*****************************
* help - onscreen help func  *
*                            *
* prints out usage and       *
* options then exits with    *
* error return code          *
*****************************/
void help(void) {
  (void)printf("\nrot13 ver 1.3 (C) 1997 Anvilsoft");
  (void)printf("\nUsage is: %s [[options] <infile>]\n", program_name);
  (void)printf("\nOptions:\n");
  (void)printf("    -h           Print this message\n");
  (void)printf("    -q           Turn quiet mode off (Print statistics)\n");
  (void)printf("    -o<filename> Specify output file. stdout is default\n");
  (void)printf("    -d           Delete input file (if specified)\n");
  (void)printf("    <infile>     Use <infile> instead of stdin for input\n\n");
  exit(2);
}
